/*---------------------------------------------------------------------------------------------------------------------------------------------------------------------
NOTE: [ I STAND ON THE SHOULDERS OF GIANTS ]

		SNS_SHI_FIBS_CHANNEL is modified from SHI_CHANNEL_TRUE_TRO_MODIFIED_VERSION by Avery T. Horton, aka TheRumpledOne, which was itself a modified version 
		of the original SHI_CHANNEL_TRUE origianlly created by Shurka & Kevin.  I adhere to the MOTTO OF TRUE PROGRAMMERS:  ** ADAPT - MODIFY - IMPROVE **
		I rarely claim any program I develop is "100% uniquely my code".  More of my and TRO's indicators can be found at www.kreslik.com.


-----------------------------------------------------------------------------------------------------------------------------------------------------------------------
SNS_SHI_FIBS_CHANNEL by Steven N. Scott (aka SNSCOTT):

Copyright (C) 2009, Steven N. Scott
mailto:steven.n.scott@gmail.com
_____________________________________________________________________________________________________________
Version History

v1.0			Initial Release																							11/12/2009
v1.1			Added CHANNEL.RAY option to make lines extend													11/12/2009
v1.1a			Fixed CHANNEL.RAY bug with FIBO lines (could not turn them off "extend" mode)			11/12/2009
v1.2			Modified to use SNS_ObjectLibrary.mqh																11/30/2009

_____________________________________________________________________________________________________________


Modification of the SHI_CHANNEL_TRUE code to add FIB CHANNELS to the SHI CHANNEL and to allow for (optionally) displaying SHI Channels from multiple timeframes 
on one chart in addition to providing a simple,optional gauge with SHI Channel data in numerical form.  

Up/Down/Sideways colors are independently adjustable by the user for each time frame.

--------------------------------------------------------------------------------------------------------------------------------------------------------------------- */

#property copyright "(c) - 2004-Shurka & Kevin / 2009-Avery T. Horton / 2009-Steven N. Scott"
#property link "steven.n.scott@gmail.com"

// THE FOLLWING INCLUDE (MQH) FILE NEEDS TO BE IN THE EXPERTS\INCLUDES FOLDER
#include <SNS_ObjectLibrary.mqh>

#property indicator_chart_window
#property indicator_buffers 2
#property indicator_color1 Pink


//____________________________________________________________________________________________________________________________________________________________
// INDICATOR CUSTOMIZABLE PROPERTIES

extern string			Chart.Symbol					= "";
extern int				Chart.Period					= 0;
extern bool   			Sound.Alert   	 				= false;

extern string			gauge.note						= "GAUGE OPTIONS--------------------";
extern bool				Gauge.Show						= false;
extern int				Gauge.Corner					= 1;
extern int				Gauge.X							= 120;
extern int				Gauge.Y							= 10;
extern int				Gauge.LineSpacing 			= 25;
extern int				Header.X							= -100;
extern int				Header.Y							= 0;
extern int				Col1.X							= 0;
extern int				Col2.X							= -90;
extern string			Header.Font						= "Verdana BOLD";
extern int				Header.FontSize				= 14;
extern color			Header.FontColor  			= RoyalBlue;
extern string			Label.Font						= "Verdana";
extern int				Label.FontSize					= 12;
extern color			Label.FontColor  				= Goldenrod;
extern string			Value.Font						= "Verdana";
extern int				Value.FontSize					= 12;
extern color			Value.FontColor  				= LightGray;

extern string			shi.note							= "SHI CHANNEL OPTIONS---------------";
extern int 				BarsForFract 					= 0;
extern int				Channel.Line.Width			= 8;
extern int				Channel.Line.Style			= 0;
extern int				Channel.MidLine.Width		= 1;
extern int				Channel.MidLine.Style		= 0;
extern bool				Channel.Ray						= true;
extern color			Channel.Color.Up				= Lime;
extern color			Channel.Color.Dn				= Red;
extern color			Channel.Color.Eq				= Yellow;

extern int				Breakout.Arrow					= 174;
extern color			Breakout.Color					= LemonChiffon;
extern int				Breakout.Size					= 5;

extern bool   			PriceBox.Show  				= false;
extern int    			PriceBox.Width    			= 3;
extern color			PriceBox.Color					= DimGray;
extern int 				AllBars 							= 240;

extern string			fibo.note						= "FIBO CHANNEL OPTIONS-------------";
extern bool				Fibo.Show						= true;
extern int				Fibo.Width						= 1;
extern int				Fibo.Style						= 0;
extern color			Fibo.Color						= DimGray;
extern color			Fibo.Level.Color				= DimGray;
extern int				Fibo.Level.Width				= 1;
extern int				Fibo.Level.Style				= 2;


extern string			multiple.note					= "MULTIPLE TIMEFRAME OPTIONS-------";
extern bool				Channel.Week.Show 			= false;
extern bool				Channel.Day.Show 				= false;
extern bool				Channel.H4.Show 				= false;
extern bool				Channel.H1.Show 				= false;

extern color			Week.Color.Up					= DarkGreen;
extern color			Week.Color.Dn					= Maroon;
extern color			Week.Color.Eq					= DarkGoldenrod;

extern color			Day.Color.Up					= Green;
extern color			Day.Color.Dn					= FireBrick;
extern color			Day.Color.Eq					= DarkOrange;

extern color			H4.Color.Up						= SeaGreen;
extern color			H4.Color.Dn						= Crimson;
extern color			H4.Color.Eq						= Orange;

extern color			H1.Color.Up						= MediumSeaGreen;
extern color			H1.Color.Dn						= Coral;
extern color			H1.Color.Eq						= C'255,255,155';



//____________________________________________________________________________________________________________________________________________________________
// BUFFERS

double ExtMapBuffer1[],slopebuffer[];


//____________________________________________________________________________________________________________________________________________________________
//INTERNAL GLOBALS

int CurrentBar = 0, B1 = -1, B2 = -1,UpDown = 0,i = 0, AB = 300, BFF = 0, ishift = 0, OldBars = -1, digits, period;
double Step = 0,P1 = 0, P2 = 0, PP = 0,iprice = 0, point=10, pips=10;
 
datetime T1, T2, Trigger;

string symbol, tag, Chart.Name;  

color Channel.Color 	= White;

color Week.Color,Day.Color,H4.Color,H1.Color;

double UpperLR, LowerLR, MiddleLR, Slope, ChannelSize; 
 

//***************************************************************************************************************************************************************
//STANDARD INDICATOR FUNCTIONS

int init() {

   if (Chart.Period==0) period = Period(); else period=Chart.Period;
   if (Chart.Symbol=="") symbol = Symbol(); else symbol=Chart.Symbol;
   Chart.Name	 =  Chart.Symbol + ":"+Time.TFToString(period) ;

   digits       =  Digits ;   
   point        =  Point ;   
   if(Digits == 5 || Digits == 3) point = point * 10 ;
   
   
   DeleteObjectsByPrefix("shict:");
    
   tag = "shict:"+period+":";
      
   SetIndexArrow(0, Breakout.Arrow);
   SetIndexBuffer(0, ExtMapBuffer1);
   SetIndexEmptyValue(0, 0.0);
   SetStyles();

   return(0);
   
}

int deinit() {
  
   DeleteObjectsByPrefix(tag);
   Comment(" ");

   return(0);
   
}


int start() {

	SetStyles();
	
	return(ShowDashboard());
   
}

//***************************************************************************************************************************************************************
//CUSTOM INDICATOR FUNCTIONS

//____________________________________________________________________________________________________________________________________________________________

void SetStyles(){

   SetIndexStyle(0, DRAW_ARROW, 0, Breakout.Size, Breakout.Color);

}


//____________________________________________________________________________________________________________________________________________________________
//DISPLAY DASHBOARD

int ShowDashboard(){

	double Slope.Week = PeriodTrend(symbol,PERIOD_W1, AllBars);  	if (Channel.Week.Show && period!=PERIOD_W1) DrawChannel(PERIOD_W1,Slope.Week);
	double Slope.Day	= PeriodTrend(symbol,PERIOD_D1 ,AllBars);		if (Channel.Day.Show  && period!=PERIOD_D1) DrawChannel(PERIOD_D1,Slope.Day);
	double Slope.H4	= PeriodTrend(symbol,PERIOD_H4,AllBars);		if (Channel.H4.Show   && period!=PERIOD_H4) DrawChannel(PERIOD_H4,Slope.H4);
	double Slope.H1	= PeriodTrend(symbol,PERIOD_H1,AllBars);		if (Channel.H1.Show   && period!=PERIOD_H1) DrawChannel(PERIOD_H1,Slope.H1);

	if(Slope.Week > 0) 	Week.Color 	= Week.Color.Up; else if(Slope.Week < 0) 	Week.Color 	= Week.Color.Dn;  else 	Week.Color 	= Week.Color.Eq; 
	if(Slope.Day > 0) 	Day.Color 	= Day.Color.Up;  else if(Slope.Day < 0) 	Day.Color 	= Day.Color.Dn;   else  Day.Color 	= Day.Color.Eq; 
	if(Slope.H4 > 0) 		H4.Color 	= H4.Color.Up;   else if(Slope.H4 < 0) 	H4.Color 	= H4.Color.Dn;  	else  H4.Color 	= H4.Color.Eq; 
	if(Slope.H1 > 0) 		H1.Color 	= H1.Color.Up;   else if(Slope.H1 < 0) 	H1.Color 	= H1.Color.Dn;  	else  H1.Color 	= H1.Color.Eq; 

	Slope=PeriodTrend(symbol,period,AllBars); 
	slopebuffer[0]=Slope;
	DrawChannel(period,Slope);
	
   //DeleteObjectsByPrefix(tag);
   
   if( Gauge.Show ) {
   
   	int row=Gauge.Y;
   	Draw_Label(tag, "header", symbol+" SHI CHANNEL", Gauge.Corner, Gauge.X+Header.X, row+Header.Y, Header.Font, Header.FontSize, Header.FontColor, false, 0);
   	row+=Gauge.LineSpacing;
   	
   	Draw_Label(tag, "Channel", "CHANNEL", Gauge.Corner, Gauge.X+Col1.X, row, Label.Font, Label.FontSize, Label.FontColor, false, 0);
   	Draw_Label(tag, "chnlsz",  DoubleToStr(ChannelSize/pips, 0)+" pips", Gauge.Corner, Gauge.X+Col2.X, row, Value.Font, Value.FontSize, Value.FontColor, false, 0);
   
   	row+=Gauge.LineSpacing;
   	Draw_Label(tag, "h1lope", "H1 SLOPE", Gauge.Corner, Gauge.X+Col1.X, row, Label.Font, Label.FontSize, Label.FontColor, false, 0);
   	Draw_Label(tag, "c1slope",  DoubleToStr(Slope.H1, 0), Gauge.Corner, Gauge.X+Col2.X, row, Value.Font, Value.FontSize, H1.Color, false, 0);
   	
   	row+=Gauge.LineSpacing;
   	Draw_Label(tag, "h4lope", "H4 SLOPE", Gauge.Corner, Gauge.X+Col1.X, row, Label.Font, Label.FontSize, Label.FontColor, false, 0);
   	Draw_Label(tag, "c4slope",  DoubleToStr(Slope.H4, 0), Gauge.Corner, Gauge.X+Col2.X, row, Value.Font, Value.FontSize, H4.Color, false, 0);
   
   
   	row+=Gauge.LineSpacing;
   	Draw_Label(tag, "DSlope", "D1 SLOPE", Gauge.Corner, Gauge.X+Col1.X, row, Label.Font, Label.FontSize, Label.FontColor, false, 0);
   	Draw_Label(tag, "dslpe",  DoubleToStr(Slope.Day, 0), Gauge.Corner, Gauge.X+Col2.X, row, Value.Font, Value.FontSize, Day.Color, false, 0);

   	row+=Gauge.LineSpacing;
   	Draw_Label(tag, "WSlope", "W1 SLOPE", Gauge.Corner, Gauge.X+Col1.X, row, Label.Font, Label.FontSize, Label.FontColor, false, 0);
   	Draw_Label(tag, "wslpe",  DoubleToStr(Slope.Week, 0), Gauge.Corner, Gauge.X+Col2.X, row, Value.Font, Value.FontSize, Week.Color, false, 0);
      
   	row+=Gauge.LineSpacing;
   	Draw_Label(tag, "lowr", "LOWER", Gauge.Corner, Gauge.X+Col1.X, row, Label.Font, Label.FontSize, Label.FontColor, false, 0);
   	Draw_Label(tag, "lowrv",  DoubleToStr(LowerLR, digits), Gauge.Corner, Gauge.X+Col2.X, row, Value.Font, Value.FontSize, Value.FontColor, false, 0);

   	row+=Gauge.LineSpacing;
   	Draw_Label(tag, "uppr", "UPPER", Gauge.Corner, Gauge.X+Col1.X, row, Label.Font, Label.FontSize, Label.FontColor, false, 0);
   	Draw_Label(tag, "upprv",  DoubleToStr(UpperLR, digits), Gauge.Corner, Gauge.X+Col2.X, row, Value.Font, Value.FontSize, Value.FontColor, false, 0);

   	row+=Gauge.LineSpacing;
   	Draw_Label(tag, "midr", "MIDDLE", Gauge.Corner, Gauge.X+Col1.X, row, Label.Font, Label.FontSize, Label.FontColor, false, 0);
   	Draw_Label(tag, "midrv",  DoubleToStr(MiddleLR, digits), Gauge.Corner, Gauge.X+Col2.X, row, Value.Font, Value.FontSize, Value.FontColor, false, 0);
  
      
   }        
           
   //if(iprice != 0) ExtMapBuffer1[ishift] = iprice;


   if ( Trigger != Time[0] &&  Sound.Alert )   {
   
     if( Close[0] >= UpperLR ) { Trigger = Time[0] ; Alert(symbol,"  ", Chart.Name, " Price at/above UpperLR "+ DoubleToStr(UpperLR ,digits)); }
     if( Close[0] <= LowerLR ) { Trigger = Time[0] ; Alert(symbol,"  ", Chart.Name, " Price at/below LowerLR "+ DoubleToStr(LowerLR ,digits)); }     
   
   }

   if(PriceBox.Show) DoBoxes();
    
   OldBars = Bars ;   

   WindowRedraw() ;

}


//____________________________________________________________________________________________________________________________________________________________
//DRAW CHANNEL FOR GIVEN PERIOD

void DrawChannel(int cperiod, int slope){

	if (cperiod!=period) switch(cperiod) {
		
			case PERIOD_W1: { if(slope > 0) Channel.Color=Week.Color.Up; 	else if(slope < 0) Channel.Color=Week.Color.Dn;	else  Channel.Color=Week.Color.Eq; 	break; }
			case PERIOD_D1: { if(slope > 0) Channel.Color=Day.Color.Up; 	else if(slope < 0) Channel.Color=Day.Color.Dn;  else  Channel.Color=Day.Color.Eq; 	break; }
			case PERIOD_H4: { if(slope > 0) Channel.Color=H4.Color.Up;		else if(slope < 0) Channel.Color=H4.Color.Dn;  	else  Channel.Color=H4.Color.Eq; 	break; }
			case PERIOD_H1: { if(slope > 0) Channel.Color=H1.Color.Up; 		else if(slope < 0) Channel.Color=H1.Color.Dn;  	else  Channel.Color=H1.Color.Eq; 	break; }
			default: { if(slope > 0) Channel.Color = Channel.Color.Up;		else if(slope < 0) Channel.Color = Channel.Color.Dn;  else  Channel.Color = Channel.Color.Eq; }
			
	} else { if(slope > 0) Channel.Color = Channel.Color.Up ; else if(slope < 0) Channel.Color = Channel.Color.Dn;  else  Channel.Color = Channel.Color.Eq; }
	
   string name=Time.TFToString(cperiod);
   
   Draw_TLine(tag, Chart.Name+"_"+name+"_TL1",  Channel.Line.Style, Channel.Line.Width, Channel.Color, T2, PP + Step*AB, T1, PP, Channel.Ray	, true, 0);
   ObjectSetText(tag+Chart.Name+"_"+name+"_TL1",name+ " SHI");
   Draw_TLine(tag, Chart.Name+"_"+name+"_TL2",  Channel.Line.Style, Channel.Line.Width, Channel.Color, T2, P2, T1, P1, Channel.Ray	, true, 0);
   ObjectSetText(tag+Chart.Name+"_"+name+"_TL2",name+ " SHI");
   Draw_TLine(tag, Chart.Name+"_"+name+"_MIDL", Channel.MidLine.Style,   Channel.MidLine.Width, Channel.Color, T2, (P2 + PP + Step*AB) / 2, T1, (P1 + PP) / 2, Channel.Ray	, true, 0);
   ObjectSetText(tag+Chart.Name+"_"+name+"_MIDL",name+ " SHI");
   
   if (Fibo.Show && cperiod==period) {
		
   	Draw_FiboChannel(tag, Chart.Name+"_FIBOCHAN", Fibo.Style, Fibo.Width, Fibo.Color, Fibo.Level.Style, Fibo.Level.Width, Fibo.Level.Color,  T2, P2, T1, P1, T2, PP + Step*AB, Channel.Ray, true, 0);
   
   }

}


//____________________________________________________________________________________________________________________________________________________________
//DRAW PRICE BOXES ON CHANNEL LINES

void DoBoxes(){

	Draw_PriceBox(tag, Chart.Name+"Upper", false, PriceBox.Width, PriceBox.Color, Time[0], UpperLR, true, 0);
	Draw_PriceBox(tag, Chart.Name+"Lower", false, PriceBox.Width, PriceBox.Color, Time[0], LowerLR, true, 0);
	Draw_PriceBox(tag,Chart.Name+"Mid", 	false, PriceBox.Width, PriceBox.Color, Time[0], MiddleLR, true, 0);
               

}

//____________________________________________________________________________________________________________________________________________________________
//DETERMINE WHAT THE SHI CHANNEL IS FOR THIS CURRENCY AND TIMEFRAME

int PeriodTrend(string pair, int tf, int tbars) {

   if((tbars == 0) || (iBars(pair,tf) < tbars))  AB = iBars(pair,tf); else AB = tbars; 
   
   if(BarsForFract > 0) BFF = BarsForFract; 
          
   	else  switch (tf){
       
        	case  PERIOD_M1: 	BFF = 12; break;
        	case  PERIOD_M5: 	BFF = 24; break; // was 48
        	case  PERIOD_M15: BFF = 24; break;
        	case  PERIOD_M30: BFF = 24; break;
        	case  PERIOD_H1: 	BFF = 12; break;
        	case  PERIOD_H4: 	BFF = 15; break;
        	case  PERIOD_D1: 	BFF = 10; break;
        	case  PERIOD_W1: 	BFF = 6;  break;
        	case  PERIOD_MN1: BFF = 6;  break;
        	default: Alert("TIMEFRAME ERROR"); 
        	return(-1); 
        	              
   }
            
   CurrentBar = 2; 
   B1 = -1; 
   B2 = -1; 
   UpDown = 0;
   while(((B1 == -1) || (B2==-1)) && (CurrentBar<AB)) {
   
          if((UpDown < 1) && (CurrentBar == iLowest(pair, tf, MODE_LOW, BFF*2 + 1, CurrentBar - BFF))) {
          
                 if(UpDown == 0) { 
                 
                     UpDown = -1; 
                     B1 = CurrentBar; 
                     P1 = iLow(pair,tf,B1);
                      
                 } else  { 
                 
                     B2 = CurrentBar; 
                     P2 = iLow(pair,tf,B2);
                     
                 }
                 
           }
           
          if((UpDown > -1) && (CurrentBar == iHighest(pair, tf, MODE_HIGH, BFF*2 + 1, CurrentBar - BFF))) {
          
                 if(UpDown == 0){ 
                 
                     UpDown = 1; 
                     B1 = CurrentBar; 
                     P1 = iHigh(pair,tf,B1);
                      
                 } else { 
                 
                     B2 = CurrentBar; 
                     P2 = iHigh(pair,tf,B2); 
                     
                 }
                 
          }
          
          CurrentBar++;
          
   }
   
   if((B1 == -1) || (B2 == -1)) {
   
       DeleteObjectsByPrefix(tag); 
       return(-1);
       
   } 
   
   Step = (P2 - P1) / (B2 - B1); 
   P1 = P1 - B1*Step; 
   B1 = 0; 
   ishift = 0; 
   iprice = 0;
   if(UpDown == 1) { 
   
          PP = iLow(pair,tf,2) - 2 * Step;
          
          for(i = 3; i <= B2; i++)  if(iLow(pair,tf,i) < PP + Step*i)  PP = iLow(pair,tf,i) - i*Step; 

          if(iLow(pair,tf,0) < PP) { ishift = 0; iprice = PP; }
          
          if(iLow(pair,tf,1) < PP + Step) { ishift = 1; iprice = PP + Step; }
          
          if(iHigh(pair,tf,0) > P1) { ishift = 0; iprice = P1;  }
          
          if(iHigh(pair,tf,1)  > P1 + Step) { ishift = 1; iprice = P1 + Step; }
          
   } else { 
   
          PP = iHigh(pair,tf,2)  - 2 * Step;
          
          for(i = 3; i <= B2; i++)  if(iHigh(pair,tf,i) > PP + Step*i) PP = iHigh(pair,tf,i) - i*Step;

          if(iLow(pair,tf,0) < P1) { ishift = 0; iprice = P1;  }
          
          if(iLow(pair,tf,1) < P1 + Step) { ishift = 1;  iprice = P1 + Step; }
          
          if(iHigh(pair,tf,0)  > PP) { ishift = 0; iprice = PP; }
          
          if(iHigh(pair,tf,1)  > PP + Step) { ishift = 1;  iprice = PP + Step; }
          
   }

   P2 = P1 + AB * Step;
   T1 = iTime(pair,tf,B1); 
   T2 = iTime(pair,tf,AB);

	ChannelSize 		= MathAbs(PP - P1) / Point ;
	double xSlope     = -Step / Point ;
	double qq1 			= NormalizeDouble(PP,digits) ;
	double qq2 			= NormalizeDouble(P1,digits) ;

   LowerLR  = MathMin(qq1,qq2) ;
   UpperLR  = MathMax(qq1,qq2) ;
   MiddleLR = NormalizeDouble((P1 + PP) / 2 ,digits) ;     
                  
              
   return(xSlope);


}